'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = void 0;

var _react = _interopRequireDefault(require('react'));

var _propTypes = _interopRequireDefault(require('prop-types'));

var _BaseTable = _interopRequireDefault(require('./BaseTable'));

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

function _extends() {
  _extends =
    Object.assign ||
    function(target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];
        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }
      return target;
    };
  return _extends.apply(this, arguments);
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly)
      symbols = symbols.filter(function(sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    keys.push.apply(keys, symbols);
  }
  return keys;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    if (i % 2) {
      ownKeys(Object(source), true).forEach(function(key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function(key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }
  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {value: value, enumerable: true, configurable: true, writable: true});
  } else {
    obj[key] = value;
  }
  return obj;
}

function BodyTable(props, _ref) {
  var manager = _ref.manager,
    baseProps = _ref.props,
    getProps = _ref.getProps;
  var columnManager = manager.columnManager,
    sizeManager = manager.sizeManager;
  var prefixCls = baseProps.prefixCls,
    fixedHeader = baseProps.fixedHeader,
    showHeader = baseProps.showHeader,
    dataSource = baseProps.dataSource,
    bodyMaxHeight = baseProps.bodyMaxHeight;
  var saveRef = props.saveRef,
    fixed = props.fixed,
    handleBodyScroll = props.handleBodyScroll,
    registerForce = props.registerForce;

  var baseTable = _react['default'].createElement(_BaseTable['default'], {
    hasHead: !fixedHeader,
    hasBody: true,
    fixed: fixed,
    registerForce: registerForce,
    columns: columnManager.headColumns(fixed)
  });

  var height = 0;

  if (dataSource && dataSource.length > 0) {
    height = sizeManager._wrapperHeight - sizeManager.footerHeight;
    height = height - (showHeader && fixedHeader ? sizeManager._headerHeight : 0);
  }

  var dataHeight = sizeManager._dataHeight;

  if (showHeader) {
    dataHeight += fixedHeader ? 0 : sizeManager._headerHeight;
  }

  var scrollSize = sizeManager.scrollSizeX();
  dataHeight = dataHeight + scrollSize;

  if (getProps('useScrollY')) {
    height = Math.min(height, dataHeight);
  } else {
    height = dataHeight;
  }

  var style = {
    height: height
  };

  if (bodyMaxHeight) {
    style.maxHeight = bodyMaxHeight;
  }

  if (scrollSize > 0 && fixed) {
    style.marginBottom = '-'.concat(height > 0 ? scrollSize : 0, 'px');
    style.paddingBottom = '0px';
  }

  var scrollRef = 'bodyTable';

  if (fixed === 'left') {
    scrollRef = 'fixedColumnsBodyLeft';
  } else if (fixed === 'right') {
    scrollRef = 'fixedColumnsBodyRight';
  }

  var bodyProps = {
    ref: saveRef(scrollRef),
    onScroll: handleBodyScroll
  };

  if (getProps('onWheel')) {
    bodyProps.onWheel = getProps('onWheel');
  }

  if (fixed) {
    fixed === 'left' && (style.width = columnManager.getWidth(fixed));

    if (fixed === 'right' && sizeManager._hasScrollY) {
      style.width = columnManager.getWidth(fixed) + sizeManager.scrollSizeY();
    }

    return _react['default'].createElement(
      'div',
      {
        key: 'bodyTable',
        className: ''.concat(prefixCls, '-body-outer'),
        style: _objectSpread({}, style)
      },
      _react['default'].createElement(
        'div',
        _extends(
          {
            className: ''.concat(prefixCls, '-body-inner'),
            style: {
              height: '100%',
              overflowY: sizeManager._hasScrollY ? 'scroll' : 'hidden',
              overflowX: sizeManager._hasScrollX ? 'scroll' : 'hidden'
            }
          },
          bodyProps
        ),
        baseTable
      )
    );
  }

  if (style.height === 0 && empty) {
    style.height = getProps('rowHeight') + scrollSize;
  }

  function empty() {
    var emptyText = baseProps.emptyText,
      dataSource = baseProps.dataSource,
      rowHeight = baseProps.rowHeight,
      prefixCls = baseProps.prefixCls,
      fixedHeader = baseProps.fixedHeader,
      showHeader = baseProps.showHeader;

    if (dataSource && dataSource.length > 0) {
      return null;
    }

    var emptyStyle = {
      height: rowHeight,
      lineHeight: rowHeight + 'px',
      flex: '0 1 '.concat(rowHeight, 'px'),
      textAlign: 'center',
      bottom: 0
    };

    if (scrollSize > 0 && fixedHeader && showHeader) {
      emptyStyle.bottom = scrollSize;
    }

    return _react['default'].createElement(
      'div',
      {
        key: 'table-empty-text',
        className: ''.concat(prefixCls, '-empty-text'),
        style: emptyStyle
      },
      typeof emptyText === 'function' ? emptyText() : emptyText
    );
  }

  return _react['default'].createElement(
    'div',
    {
      key: 'bodyTable',
      className: ''.concat(prefixCls, '-body'),
      style: style
    },
    _react['default'].createElement(
      'div',
      _extends(
        {
          style: {
            height: '100%',
            overflowY: sizeManager._hasScrollY ? 'scroll' : 'hidden',
            overflowX: sizeManager._hasScrollX ? 'scroll' : 'hidden'
          }
        },
        bodyProps
      ),
      baseTable,
      empty()
    )
  );
}

var _default = BodyTable;
exports['default'] = _default;
BodyTable.contextTypes = {
  props: _propTypes['default'].object,
  manager: _propTypes['default'].object,
  getProps: _propTypes['default'].func
};
