'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = void 0;

var _react = _interopRequireDefault(require('react'));

var _classnames = _interopRequireDefault(require('classnames'));

var _utils = require('./utils');

var _Sorter = _interopRequireDefault(require('./Sorter'));

var _types = require('./types');

var _Sortable = _interopRequireDefault(require('./Sortable'));

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly)
      symbols = symbols.filter(function(sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    keys.push.apply(keys, symbols);
  }
  return keys;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    if (i % 2) {
      ownKeys(Object(source), true).forEach(function(key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function(key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }
  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {value: value, enumerable: true, configurable: true, writable: true});
  } else {
    obj[key] = value;
  }
  return obj;
}

function HeadCell(props) {
  var key = props.key,
    column = props.column,
    _props$current = props.current,
    current = _props$current === void 0 ? 0 : _props$current,
    prefixCls = props.prefixCls,
    headerRowHeight = props.headerRowHeight,
    orders = props.orders,
    onSort = props.onSort,
    fixed = props.fixed,
    onHeaderRow = props.onHeaderRow,
    onDrag = props.onDrag,
    SelectIcon = props.SelectIcon;
  var children = column.children || [];
  var dataIndex = column.dataIndex,
    align = column.align,
    title = column.title,
    sortEnable = column.sortEnable,
    onHeaderCell = column.onHeaderCell;
  var rowSpan = column[_types.CS.rowSpan];
  var width = column[_types.CS._width];
  var order = orders[dataIndex];
  var style = (0, _utils.cellAlignStyle)(align);

  if (width) {
    style.width = width;
    style.minWidth = width;
  } else {
    style.flex = 1;
  }

  style.height = (rowSpan || 1) * headerRowHeight;
  style.lineHeight = ''.concat(style.height - 1, 'px');

  if (onHeaderCell) {
    style = _objectSpread({}, style, {}, onHeaderCell(column));
  }

  var cellProps = {
    className: (0, _classnames['default'])('th', {
      'has-child': children.length > 0
    }),
    style: style,
    fixed: fixed
  };

  if (onHeaderRow) {
    cellProps = _objectSpread({}, cellProps, {}, onHeaderRow(column));
  }

  if (children.length === 0) {
    cellProps.key = key;
  }

  var text = title;

  if (typeof title === 'function') {
    text = title(column);
  }

  var sorter;

  if (sortEnable && children.length === 0 && order) {
    sorter = (0, _Sorter['default'])({
      dataIndex: dataIndex,
      order: order,
      prefixCls: prefixCls,
      onSort: onSort
    });
  }

  if (sortEnable && children.length === 0) {
    cellProps.onClick = function() {
      return onSort(column.dataIndex, order === 'desc' || order === true ? 'asc' : 'desc');
    };
  }

  var cell = _react['default'].createElement('div', cellProps, SelectIcon, text, sorter);

  if (children.length > 0) {
    var _style = {};

    if (current !== 0) {
      _style.display = 'inline-block';
      _style['float'] = 'left';
    }

    return _react['default'].createElement(
      'div',
      {
        className: current === 0 ? 'row-group' : '',
        key: key,
        style: _style,
        fixed: fixed
      },
      cell,
      _react['default'].createElement(
        'div',
        {
          className: 'col-group'
        },
        _react['default'].createElement(
          _Sortable['default'],
          {
            parent: column,
            columns: children,
            onDrag: onDrag
          },
          children.map(function(child, i) {
            return HeadCell({
              key: ''
                .concat(key, '-')
                .concat(current, '-')
                .concat(i),
              column: child,
              headerRowHeight: headerRowHeight,
              current: current + 1,
              orders: orders,
              prefixCls: prefixCls,
              onSort: onSort,
              fixed: fixed,
              onDrag: onDrag
            });
          })
        )
      )
    );
  }

  return cell;
}

var _default = HeadCell;
exports['default'] = _default;
