'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = void 0;

var _react = _interopRequireDefault(require('react'));

var _max = _interopRequireDefault(require('lodash/max'));

var _multiply = _interopRequireDefault(require('lodash/multiply'));

var _isNumber = _interopRequireDefault(require('lodash/isNumber'));

var _toNumber = _interopRequireDefault(require('lodash/toNumber'));

var _divide = _interopRequireDefault(require('lodash/divide'));

var _floor = _interopRequireDefault(require('lodash/floor'));

var _sumBy = _interopRequireDefault(require('lodash/sumBy'));

var _isNaN = _interopRequireDefault(require('lodash/isNaN'));

var _min = _interopRequireDefault(require('lodash/min'));

var _types = require('../types');

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly)
      symbols = symbols.filter(function(sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    keys.push.apply(keys, symbols);
  }
  return keys;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    if (i % 2) {
      ownKeys(Object(source), true).forEach(function(key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function(key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }
  return target;
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _nonIterableSpread() {
  throw new TypeError('Invalid attempt to spread non-iterable instance');
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === '[object Arguments]')
    return Array.from(iter);
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) {
      arr2[i] = arr[i];
    }
    return arr2;
  }
}

function _typeof(obj) {
  '@babel/helpers - typeof';
  if (typeof Symbol === 'function' && typeof Symbol.iterator === 'symbol') {
    _typeof = function _typeof(obj) {
      return typeof obj;
    };
  } else {
    _typeof = function _typeof(obj) {
      return obj && typeof Symbol === 'function' && obj.constructor === Symbol && obj !== Symbol.prototype
        ? 'symbol'
        : typeof obj;
    };
  }
  return _typeof(obj);
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError('Cannot call a class as a function');
  }
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {value: value, enumerable: true, configurable: true, writable: true});
  } else {
    obj[key] = value;
  }
  return obj;
}

var _path = _types.CS._path,
  _pathKey = _types.CS._pathKey,
  _currentRow = _types.CS._currentRow,
  _width = _types.CS._width,
  _minWidth = _types.CS._minWidth,
  path = _types.CS.path,
  rowSpan = _types.CS.rowSpan,
  colSpan = _types.CS.colSpan;
var percentReg = /^\d+\.?\d{0,2}%$/;

var ColumnManager = function ColumnManager(_ref) {
  var _this = this;

  var _columns = _ref.columns,
    _minWidth2 = _ref.minWidth,
    _headerRowHeight = _ref.headerRowHeight;

  _classCallCheck(this, ColumnManager);

  _defineProperty(this, '_cached', {});

  _defineProperty(this, '_maxRowSpan', 1);

  _defineProperty(this, 'init', function() {
    _this.width = 0;
    _this.leftWidth = 0;
    _this.rightWidth = 0;
    _this._maxRowSpan = 1;
  });

  _defineProperty(this, 'isAnyColumnsLeftFixed', function() {
    return _this._cache('isAnyColumnsLeftFixed', function() {
      return _this.groupedColumns().some(function(column) {
        return column.fixed === 'left' || column.fixed === true;
      });
    });
  });

  _defineProperty(this, 'isAnyColumnsRightFixed', function() {
    return _this._cache('isAnyColumnsRightFixed', function() {
      return _this.groupedColumns().some(function(column) {
        return column.fixed === 'right';
      });
    });
  });

  _defineProperty(this, 'headColumns', function(fixed) {
    if (fixed === 'left') {
      return _this.leftColumns();
    } else if (fixed === 'right') {
      return _this.rightColumns();
    }

    return _this.groupedColumns();
  });

  _defineProperty(this, 'bodyColumns', function(fixed) {
    if (fixed === 'left') {
      return _this.leftLeafColumns();
    } else if (fixed === 'right') {
      return _this.rightLeafColumns();
    }

    return _this.leafColumns();
  });

  _defineProperty(this, 'leftColumns', function() {
    return _this._cache('leftColumns', function() {
      return _this.groupedColumns().filter(function(column) {
        return column.fixed === 'left' || column.fixed === true;
      });
    });
  });

  _defineProperty(this, 'rightColumns', function() {
    return _this._cache('rightColumns', function() {
      return _this.groupedColumns().filter(function(column) {
        return column.fixed === 'right';
      });
    });
  });

  _defineProperty(this, 'leafColumns', function() {
    return _this._cache('leafColumns', function() {
      return _this._leafColumns(_this.groupedColumns());
    });
  });

  _defineProperty(this, 'leftLeafColumns', function() {
    return _this._cache('leftLeafColumns', function() {
      return _this._leafColumns(_this.leftColumns());
    });
  });

  _defineProperty(this, 'rightLeafColumns', function() {
    return _this._cache('rightLeafColumns', function() {
      return _this._leafColumns(_this.rightColumns());
    });
  });

  _defineProperty(this, 'groupedColumns', function() {
    return _this._cache('groupedColumns', function() {
      return _this._updateWidth(_this._groupColumns(_this.columns));
    });
  });

  _defineProperty(this, 'headerSize', function() {
    return {
      _leftWidth: _this.leftWidth,
      _rightWidth: _this.rightWidth,
      _headerWidth: _this.width,
      _hasScrollX: _this.hasOverflowX,
      _headerHeight: _this._maxRowSpan * _this.headerRowHeight
    };
  });

  _defineProperty(this, 'updateWidth', function(wrapperWidth) {
    if (_this.wrapperWidth !== wrapperWidth) {
      _this.wrapperWidth = wrapperWidth;
      _this._cached = {};

      _this.init();

      _this.groupedColumns();
    }

    return _this.headerSize();
  });

  _defineProperty(this, 'getWidth', function(fixed) {
    if (fixed === 'left' || fixed === true) {
      return _this.leftWidth;
    } else if (fixed === 'right') {
      return _this.rightWidth;
    }

    return _this.width;
  });

  _defineProperty(this, 'reset', function(_ref2) {
    var columns = _ref2.columns,
      minWidth = _ref2.minWidth,
      headerRowHeight = _ref2.headerRowHeight;
    _this.columns = columns;
    _this.minWidth = minWidth;
    _this.headerRowHeight = headerRowHeight;
    _this._cached = {};

    _this.init();

    _this.groupedColumns();

    return _this.headerSize();
  });

  _defineProperty(this, 'updateGroupedColumns', function(columns) {
    _this._cached = {};
    _this.columns = _this._uColumns(columns);
    _this._cached['groupedColumns'] = columns;
  });

  _defineProperty(this, '_uColumns', function(columns) {
    return columns.map(function(c) {
      var column = {};

      for (var key in c) {
        if (_typeof(key) !== 'symbol' && Object.prototype.hasOwnProperty.call(c, key)) {
          column[key] = c[key];
        }
      }

      var children = column.children || [];

      if (children.length > 0) {
        column.children = _this._uColumns(children);
      }

      return column;
    });
  });

  _defineProperty(this, '_calcWidth', function(width, wrapperWidth) {
    if (typeof width === 'string' && percentReg.test(width)) {
      var i = width.replace('%', '');
      return (0, _floor['default'])(
        _this._minWidth((0, _multiply['default'])(wrapperWidth, (0, _divide['default'])(i, 100)))
      );
    }

    if (width && typeof width === 'string') {
      width = width.replace(/[^\d]/g, '');
    }

    var ww = (0, _toNumber['default'])(width);

    if (!(0, _isNaN['default'])(ww) && (0, _isNumber['default'])(ww)) {
      return (0, _floor['default'])(_this._minWidth(ww));
    }

    return (0, _floor['default'])(_this._minWidth(width));
  });

  _defineProperty(this, '_minWidth', function(width) {
    return !width ? width : (0, _max['default'])([_this.minWidth, width]);
  });

  _defineProperty(this, '_cache', function(name, fn) {
    if (name in _this._cached) {
      return _this._cached[name];
    }

    _this._cached[name] = fn();
    return _this._cached[name];
  });

  _defineProperty(this, '_leafColumns', function(columns) {
    var leafColumns = [];

    for (var i = 0; i < columns.length; i++) {
      var column = columns[i];

      if (!column.children) {
        leafColumns.push(column);
      } else {
        leafColumns.push.apply(leafColumns, _toConsumableArray(_this._leafColumns(column.children)));
      }
    }

    return leafColumns;
  });

  _defineProperty(this, '_groupColumns', function(columns) {
    var currentRow = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var parentColumn = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var rows = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
    rows[currentRow] = rows[currentRow] || [];
    var grouped = [];

    var setRowSpan = function setRowSpan(column) {
      var rs = rows.length - currentRow;

      if (column && !column.children && rs > 1 && (!column[rowSpan] || column[rowSpan] < rs)) {
        column[rowSpan] = rs;
      }
    };

    for (var index = 0; index < columns.length; index++) {
      var column = columns[index];

      var newColumn = _objectSpread({}, column);

      rows[currentRow].push(newColumn);
      newColumn[path] = [].concat(_toConsumableArray(parentColumn[path] || [currentRow]), [index]);
      newColumn[_pathKey] = newColumn[path].join('-');
      newColumn[_currentRow] = currentRow;
      parentColumn[colSpan] = parentColumn[colSpan] || 0;

      if (newColumn.children && newColumn.children.length > 0) {
        newColumn.children = _this._groupColumns(newColumn.children, currentRow + 1, newColumn, rows);
        parentColumn[colSpan] = parentColumn[colSpan] + newColumn[colSpan];
        newColumn[_width] = (0, _sumBy['default'])(newColumn.children, _width);
      } else {
        parentColumn[colSpan]++;
        newColumn[_width] = _this._calcWidth(newColumn.width, _this.wrapperWidth);

        if ((0, _isNumber['default'])(newColumn.minWidth) && !(0, _isNaN['default'])(newColumn.minWidth)) {
          newColumn[_width] = (0, _max['default'])([newColumn[_width], newColumn.minWidth]);
        }

        if ((0, _isNumber['default'])(newColumn.maxWidth) && !(0, _isNaN['default'])(newColumn.maxWidth)) {
          newColumn[_width] = (0, _min['default'])([newColumn[_width], newColumn.maxWidth]);
        }
      }

      newColumn[_minWidth] = newColumn[_width];

      for (var i = 0; i < rows[currentRow].length - 1; ++i) {
        setRowSpan(rows[currentRow][i]);
      }

      if (index + 1 === columns.length) {
        setRowSpan(newColumn);
      }

      if (_this._maxRowSpan < currentRow + 1) {
        _this._maxRowSpan = currentRow + 1;
      }

      grouped.push(newColumn);
    }

    _this.hasOverflowX = _this.width > _this.wrapperWidth && _this.wrapperWidth !== 0;
    return grouped;
  });

  _defineProperty(this, '_updateWidth', function(columns) {
    var wrapperWidth = _this.wrapperWidth || 0;
    var leftColumns = columns.filter(function(c) {
      return c.fixed === true || c.fixed === 'left';
    });
    var rightColumns = columns.filter(function(c) {
      return c.fixed === 'right';
    });

    var leafColumns = _this._leafColumns(columns);

    var leftLeafColumns = _this._leafColumns(leftColumns);

    var rightLeafColumns = _this._leafColumns(rightColumns);

    var len = leafColumns.length - leftLeafColumns.length - rightLeafColumns.length;
    var last;
    var lcArr =
      leafColumns.filter(function(c) {
        return !c.fixed && c.fixed !== 'left' && c.fixed !== 'right';
      }) || [];

    if (leafColumns.length > 0) {
      last = leafColumns[leafColumns.length - 1];
    }

    if (lcArr.length > 0) {
      last = lcArr[lcArr.length - 1];
    }

    var baseWidth = (0, _sumBy['default'])(columns, _width);
    _this.width = 0;
    var centerWidth = 0;
    _this.leftWidth = (0, _sumBy['default'])(leftColumns, _width);
    _this.rightWidth = (0, _sumBy['default'])(rightColumns, _width);
    var average = (0, _max['default'])([(0, _floor['default'])((wrapperWidth - baseWidth) / len), 0]);
    var cols = [];

    var update = function update(columns) {
      var fixed = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      return columns.map(function(column) {
        var width = column[_width];
        var children = column.children || [];

        if (children.length > 0) {
          column.children = update(children);
          column[_width] = (0, _sumBy['default'])(column.children, _width);

          if (cols.length > 0) {
            cols.push(column);
          }
        } else {
          if (
            !leftColumns.some(function(c) {
              return c[_pathKey] === column[_pathKey];
            }) &&
            !rightColumns.some(function(c) {
              return c[_pathKey] === column[_pathKey];
            })
          ) {
            column[_width] = width + average;
          }

          if ((0, _isNumber['default'])(column.maxWidth) && !(0, _isNaN['default'])(column.maxWidth)) {
            column[_width] = (0, _min['default'])([column.maxWidth, column[_width]]);
          }

          if (last && last[_pathKey] === column[_pathKey]) {
            cols.push(column);
          }
        }

        if (column[_currentRow] === 0) {
          _this.width += column[_width];

          if (column.fixed === 'left' || column.fixed === true) {
          } else if (column.fixed === 'right') {
          } else {
            centerWidth += column[_width];
          }
        }

        return column;
      });
    };

    var result = update(columns);

    if (last) {
      last[_width] = (0, _max['default'])([
        wrapperWidth - _this.leftWidth - _this.rightWidth - centerWidth + last[_width],
        last[_width]
      ]);

      if (cols.length > 1) {
        cols = cols.slice(1);
        cols.forEach(function(c) {
          var children = c.children || [];

          if (children.length > 0) {
            c[_width] = (0, _sumBy['default'])(children, _width);
          }
        });
      }
    }

    _this.hasOverflowX = _this.width > wrapperWidth;
    return result;
  });

  this.columns = _columns;
  this.minWidth = _minWidth2;
  this.headerRowHeight = _headerRowHeight;
  this.wrapperWidth = 0;
  this.hasOverflowX = false;
  this.init();
};

exports['default'] = ColumnManager;
