'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = void 0;

var _react = _interopRequireDefault(require('react'));

var _classnames = _interopRequireDefault(require('classnames'));

var _findIndex = _interopRequireDefault(require('lodash/findIndex'));

var _types = require('../types');

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _nonIterableSpread() {
  throw new TypeError('Invalid attempt to spread non-iterable instance');
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === '[object Arguments]')
    return Array.from(iter);
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) {
      arr2[i] = arr[i];
    }
    return arr2;
  }
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError('Cannot call a class as a function');
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ('value' in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {value: value, enumerable: true, configurable: true, writable: true});
  } else {
    obj[key] = value;
  }
  return obj;
}

var DataManager =
  /*#__PURE__*/
  (function() {
    function DataManager(_ref) {
      var _this = this;

      var getProps = _ref.getProps;

      _classCallCheck(this, DataManager);

      _defineProperty(this, '_cached', {});

      _defineProperty(this, '_bodyHeight', 0);

      _defineProperty(this, '_hasExpanded', false);

      _defineProperty(this, '_data', []);

      _defineProperty(this, 'getData', function() {
        return _this._cache('getData', function() {
          return _this._getData(_this._data);
        });
      });

      _defineProperty(this, 'getFixedData', function() {
        return _this._cache('getFixedData', function() {
          return _this._getFixedData(_this.showData());
        });
      });

      _defineProperty(this, 'showData', function() {
        return _this._cache('showData', function() {
          return _this._showData(_this.getData());
        });
      });

      _defineProperty(this, 'fixedData', function() {
        return _this._cache('fixedData', function() {
          return _this._getFixedData(_this.showData());
        });
      });

      _defineProperty(this, 'fixedDataLength', function() {
        return _this._cache('fixedDataLength', function() {
          return _this.fixedData().length;
        });
      });

      _defineProperty(this, 'isExpanded', function() {
        return _this._cache('isExpanded', function() {
          return _this.getData().some(function(d) {
            return d.children && d.children.length > 0;
          });
        });
      });

      _defineProperty(this, 'rowIsExpanded', function(record) {
        return _this.expandedRowKeys.indexOf(record[_types.DS._key]) > -1;
      });

      _defineProperty(this, 'isFixed', function(record) {
        if ((record.children || []).length > 0) {
          return false;
        }

        return record['isFixed'] === true || record['isFixed'] === 'top' || record['isFixed'] === 'bottom';
      });

      _defineProperty(this, 'reset', function(data) {
        var _data = data || [];

        _this._hasExpanded = _data.some(function(d) {
          return !_this.isFixed(d) && (d.children || []).length > 0;
        });

        if (!_this.getProps('fixedHeader')) {
          _this._data = _data;
        } else {
          var dataBase = _data.filter(function(d) {
            var children = d.children || [];
            return children.length > 0 || !_this.isFixed(d);
          });

          var topData = _data.filter(function(d) {
            var children = d.children || [];
            return (d['isFixed'] === true || d['isFixed'] === 'top') && children.length === 0;
          });

          var bottomData = _data.filter(function(d) {
            var children = d.children || [];
            return d['isFixed'] === 'bottom' && children.length === 0;
          });

          _this._data = [].concat(
            _toConsumableArray(topData),
            _toConsumableArray(dataBase),
            _toConsumableArray(bottomData)
          );
        }

        _this._bodyHeight = 0;
        _this._cached = {};

        _this.showData();
      });

      _defineProperty(this, 'getByKey', function(key) {
        var index = (0, _findIndex['default'])(_this.showData(), function(d) {
          return d[_types.DS._key] === key;
        });

        if (index < 0) {
          return null;
        }

        return _this.showData()[index];
      });

      _defineProperty(this, 'resetExpandedRowKeys', function(keys) {
        _this.expandedRowKeys = Array.from(new Set(keys));
        delete _this._cached['getRowsHeight'];
        delete _this._cached['showData'];

        _this.showData();

        return _this.expandedRowKeys;
      });

      _defineProperty(this, 'expanded', function(key) {
        var keys = _this.expandedRowKeys || [];

        if (keys.indexOf(key) < 0) {
          keys.push(key);
        } else {
          keys = keys.filter(function(k) {
            return key !== k;
          });
        }

        _this.expandedRowKeys = Array.from(new Set(keys));
        delete _this._cached['getRowsHeight'];
        delete _this._cached['showData'];

        _this.showData();

        return _this.expandedRowKeys;
      });

      _defineProperty(this, '_cache', function(name, fn) {
        if (name in _this._cached) {
          return _this._cached[name];
        }

        _this._cached[name] = fn();
        return _this._cached[name];
      });

      _defineProperty(this, '_getData', function(dataSource) {
        var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
        var parentPath = arguments.length > 2 ? arguments[2] : undefined;
        dataSource = dataSource || [];

        var getRowHeight = _this.getProps('getRowHeight');

        var rowHeight = _this.getProps('rowHeight');

        for (var index = 0; index < dataSource.length; index++) {
          var height = getRowHeight(dataSource[index], index) * rowHeight;
          var path = ''.concat(parentPath === undefined ? index : ''.concat(parentPath, '-').concat(index));
          var record = dataSource[index];
          var children = record['children'] || [];
          record[_types.DS._index] = index;
          record[_types.DS._path] = path;
          record[_types.DS._expandedLevel] = level;
          record[_types.DS._height] = height;
          record[_types.DS._key] = _this._rowKey(record, index);
          record[_types.DS._expandedEnable] = children.length > 0;

          if (_this.isFixed(record)) {
            record[_types.DS._isFixed] = record['isFixed'] === true ? 'top' : record['isFixed'];
          }

          if (children.length > 0) {
            record['children'] = _this._getData(children, level + 1, path);
          }
        }

        return dataSource;
      });

      _defineProperty(this, '_getFixedData', function(data) {
        return data.filter(function(d) {
          return d['isFixed'] && (d.children || []).length === 0;
        });
      });

      _defineProperty(this, '_showData', function(dataSource, data) {
        dataSource = dataSource || [];
        data = data || [];

        if (_this._hasExpanded) {
          if (data.length === 0) {
            _this._bodyHeight = 0;
          }
        }

        var expandedKeys = _this.expandedRowKeys || [];

        for (var index = 0; index < dataSource.length; index++) {
          var record = dataSource[index];
          record[_types.DS._showIndex] = _this._hasExpanded ? data.length : index;
          record[_types.DS._top] = _this._bodyHeight;
          record[_types.DS._rowClassName] = _this._rowClassName(record, record[_types.DS._showIndex]);
          _this._bodyHeight += record[_types.DS._height];
          var children = record.children || [];

          var _expanded =
            expandedKeys.length > 0 && expandedKeys.indexOf(record[_types.DS._key]) > -1 && children.length > 0;

          record[_types.DS._expanded] = _expanded;
          _this._hasExpanded && data.push(record);

          if (_expanded) {
            data = _this._showData(children, data);
          }
        }

        return _this._hasExpanded ? data : dataSource;
      });

      _defineProperty(this, '_rowKey', function(record, index) {
        var rowKey = _this.getProps('rowKey');

        if (typeof rowKey === 'function') {
          return String(rowKey(record, index));
        } else if (typeof rowKey === 'string') {
          return String(record[rowKey]);
        } else if (record['key']) {
          return String(record['key']);
        }

        return String(record[_types.DS._path]);
      });

      _defineProperty(this, '_rowClassName', function(record, index, level) {
        var _classNames;

        var className = '';

        var rowClassName = _this.getProps('rowClassName');

        if (typeof rowClassName === 'function') {
          className = rowClassName(record, index);
        } else if (rowClassName === 'string') {
          className = rowClassName || '';
        }

        var prefixCls = _this.getProps('prefixCls');

        return (0,
        _classnames[
          'default'
        ])('tr', ''.concat(prefixCls, '-row'), className, ((_classNames = {}), _defineProperty(_classNames, ''.concat(prefixCls, '-expanded-row-').concat(level), _this._hasExpanded), _defineProperty(_classNames, ''.concat(prefixCls, '-row-fixed'), !!record[_types.DS._isFixed]), _classNames));
      });

      this.getProps = getProps;
      this.expandedRowKeys = getProps('expandedRowKeys') || [];
      this.reset(getProps('dataSource'));
    }

    _createClass(DataManager, [
      {
        key: 'isEmpty',
        value: function isEmpty() {
          return this.getData().length === 0;
        }
      }
    ]);

    return DataManager;
  })();

exports['default'] = DataManager;
