'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.measureScrollbar = measureScrollbar;
exports.cellAlignStyle = cellAlignStyle;
exports.isInvalidRenderCellText = isInvalidRenderCellText;
exports.has = has;
exports.keys = keys;

var _react = _interopRequireDefault(require('react'));

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

var scrollbarSize;
var scrollbarMeasure = {
  position: 'absolute',
  top: '-9999px',
  width: '50px',
  height: '50px'
};
var scrollbarVerticalSize;
var scrollbarHorizontalSize;

function measureScrollbar() {
  var direction = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'vertical';

  if (typeof document === 'undefined' || typeof window === 'undefined') {
    return 0;
  }

  var isVertical = direction === 'vertical';

  if (isVertical && scrollbarVerticalSize) {
    return scrollbarVerticalSize;
  } else if (!isVertical && scrollbarHorizontalSize) {
    return scrollbarHorizontalSize;
  }

  var scrollDiv = document.createElement('div');
  Object.keys(scrollbarMeasure).forEach(function(scrollProp) {
    scrollDiv.style[scrollProp] = scrollbarMeasure[scrollProp];
  }); // Append related overflow style

  if (isVertical) {
    scrollDiv.style.overflowY = 'scroll';
  } else {
    scrollDiv.style.overflowX = 'scroll';
  }

  document.body.appendChild(scrollDiv);
  var size = 0;

  if (isVertical) {
    size = scrollDiv.offsetWidth - scrollDiv.clientWidth;
    scrollbarVerticalSize = size;
  } else {
    size = scrollDiv.offsetHeight - scrollDiv.clientHeight;
    scrollbarHorizontalSize = size;
  }

  document.body.removeChild(scrollDiv);
  return size;
}

function cellAlignStyle(align) {
  var style = {};

  if (align) {
    switch (align) {
      case 'center':
        style.justifyContent = 'center';
        style.textAlign = 'center';
        break;

      case 'left':
        style.justifyContent = 'flex-start';
        style.textAlign = 'left';
        break;

      case 'right':
        style.justifyContent = 'flex-end';
        style.textAlign = 'right';
        break;
    }
  }

  return style;
}

function isInvalidRenderCellText(text) {
  return text && !_react['default'].isValidElement(text) && Object.prototype.toString.call(text) === '[object Object]';
}

function has(obj, key) {
  return Object.prototype.hasOwnProperty.call(obj || {}, key);
}

function keys(obj) {
  return Object.keys(obj || {}) || [];
}
